@@ -7,7 +7,7 @@ Pyramid authn policy that ties together multiple backends.
 
 __ver_major__ = 0
 __ver_minor__ = 3
-__ver_patch__ = 1
+__ver_patch__ = 2
 __ver_sub__ = ""
 __ver_tuple__ = (__ver_major__, __ver_minor__, __ver_patch__, __ver_sub__)
 __version__ = "%d.%d.%d%s" % __ver_tuple__
@@ -110,13 +110,19 @@ class MultiAuthenticationPolicy(object):
         for policy in self._policies:
             principals.update(policy.effective_principals(request))
         if self._callback is not None:
-            userid = self.unauthenticated_userid(request)
-            if userid is not None:
+            principals.discard(Authenticated)
+            groups = None
+            for policy in self._policies:
+                userid = policy.authenticated_userid(request)
+                if userid is None:
+                    continue
                 groups = self._callback(userid, request)
                 if groups is not None:
-                    principals.add(userid)
-                    principals.add(Authenticated)
-                    principals.update(groups)
+                    break
+            if groups is not None:
+                principals.add(userid)
+                principals.add(Authenticated)
+                principals.update(groups)
         return list(principals)
 
     def remember(self, request, principal, **kw):
